from __future__ import absolute_import

from django.conf.urls import url

from .views import LiveWatchView


urlpatterns = [
    url(r'^$', LiveWatchView.as_view(), name='livewatch'),
    url(r'^(?P<service>\w+)/$', LiveWatchView.as_view(), name='livewatch-service'),
]



from __future__ import absolute_import

import django_rq
from django.core.cache import cache
from django.utils import timezone

from .base import TaskExtension


def livewatch_update_rq_task(key):
    return cache.set(key, timezone.now(), 2592000)


class RqExtension(TaskExtension):
    name = 'rq'

    def run_task(self):
        key = 'livewatch_{0}'.format(self.name)
        django_rq.enqueue(livewatch_update_rq_task, key)



from __future__ import absolute_import

import re
import logging

from django.http import HttpResponse, HttpResponseNotFound
from django.views.generic import View

from .utils import get_extensions


logger = logging.getLogger(__name__)

KEY_RE = re.compile(r'^[a-f0-9]{32}$')


class LiveWatchView(View):

    def get(self, request, *args, **kwargs):
        extensions = get_extensions()

        services_failed = []
        service = kwargs.get('service', None)
        if service:
            if service not in extensions:
                return HttpResponseNotFound()

            if not extensions[service].check_service(request):
                services_failed.append(service)
        else:
            for service in extensions:
                if not extensions[service].check_service(request):
                    services_failed.append(service)

        if services_failed:
            return HttpResponseNotFound('Error {0}'.format(', '.join(sorted(services_failed))))

        if 'key' not in request.GET:
            return HttpResponse('Ok')

        if KEY_RE.match(request.GET['key']):
            return HttpResponse(request.GET['key'])

        return HttpResponseNotFound()



import importlib
import logging

from django.conf import settings


logger = logging.getLogger(__name__)


def get_extensions(reload_extensions=False):
    if not hasattr(get_extensions, '_cache') or reload_extensions:
        extensions = {}
        registered = getattr(settings, 'LIVEWATCH_EXTENSIONS', [])

        for extension in registered:
            try:
                base, cls = extension.split(':', 1)
                module = importlib.import_module(base)
                extension_class = getattr(module, cls)
                extensions[extension_class.name] = extension_class()
            except ImportError:
                logger.error('Failed to import {0}'.format(extension))

        get_extensions._cache = extensions

    return get_extensions._cache



# -*- coding: utf-8 -*-
import sys
import os
import pkg_resources

try:
    import sphinx_rtd_theme
except ImportError:
    sphinx_rtd_theme = None

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'livewatch.tests.settings')

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx'
]

templates_path = ['_templates']

source_suffix = '.rst'

master_doc = 'index'

# General information about the project.
project = u'django-livewatch'
copyright = u'2015, Moccu GmbH & Co. KG'

distribution = pkg_resources.get_distribution('django-livewatch')

version = distribution.version
release = version

exclude_patterns = []

pygments_style = 'sphinx'

html_domain_indices = True

if sphinx_rtd_theme:
    html_theme = 'sphinx_rtd_theme'
    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
else:
    html_theme = 'default'

html_static_path = ['_static']

htmlhelp_basename = 'livewatchdoc'


latex_elements = {}

latex_documents = [
  ('index', 'livewatch.tex', u'livewatch Documentation',
   u'Moccu GmbH \\& Co. KG', 'manual'),
]

man_pages = [
    ('index', 'livewatch', u'livewatch Documentation',
     [u'Moccu GmbH & Co. KG'], 1)
]

texinfo_documents = [
  ('index', 'livewatch', u'livewatch Documentation',
   u'Moccu GmbH & Co. KG', 'livewatch', 'One line description of project.',
   'Miscellaneous'),
]

intersphinx_mapping = {
    'python': ('http://docs.python.org/', None),
    'django': (
        'http://docs.djangoproject.com/en/dev/',
        'http://docs.djangoproject.com/en/dev/_objects/'
    ),
}



import logging

from django.core.cache import cache

from .base import BaseExtension

logger = logging.getLogger(__name__)


class CacheExtension(BaseExtension):
    name = 'cache'

    def check_service(self, request=None):
        key = 'livewatch_{0}'.format(self.name)
        cache.set(key, 'cache_activated', 2592000)

        cache_activated = cache.get(key) == 'cache_activated'
        cache.delete(key)

        return cache_activated



import os
import sys
import codecs
from setuptools import setup, find_packages


version = '0.2.5'


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    print('You probably want to also tag the version now:')
    print('  git tag -a %s -m "version %s"' % (version, version))
    print('  git push --tags')
    sys.exit()


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


install_requirements = [
    'django>=1.5',
]


test_requirements = [
    'py==1.4.26',
    'pyflakes==0.8.1',
    'pytest==2.6.4',
    'pytest-cache==1.0',
    'pytest-cov==1.8.1',
    'pytest-flakes==0.2',
    'pytest-pep8==1.0.6',
    'pytest-django==2.8.0',
    'cov-core==1.15.0',
    'coverage==3.7.1',
    'execnet==1.3.0',
    'pep8==1.6.2',
    'factory-boy==2.4.1',
    'mock==1.0.1',
    'celery==3.1.16',
    'django-rq==0.7.0',
    'django-redis-cache==0.13.0',
    'hiredis==0.1.6',
]


# sdist
if not 'bdist_wheel' in sys.argv:
    try:
        import importlib
    except ImportError:
        install_requirements.append('importlib>=1.0.3')


# bdist_wheel
extras_require = {
    # http://wheel.readthedocs.org/en/latest/#defining-conditional-dependencies
    ':python_version=="2.6"': ['importlib>=1.0.3'],
    'tests': test_requirements,
    'rq': ['django-rq'],
    'celery': ['celery'],
}


setup(
    name='django-livewatch',
    version=version,
    description=(
        'django-livewatch integrates livewatch.de for django projects'),
    long_description=read('README.rst'),
    author='Moccu GmbH & Co. KG',
    author_email='info@moccu.com',
    url='https://github.com/moccu/django-livewatch/',
    packages=find_packages(exclude=[
        'livewatch.tests',
    ]),
    install_requires=install_requirements,
    extras_require=extras_require,
    include_package_data=True,
    license='BSD',
    keywords=['livewatch', 'django'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    zip_safe=False,
)



from __future__ import absolute_import

from django.core.cache import cache
from django.utils import timezone

from celery import shared_task


@shared_task
def livewatch_update_celery_task(key):
    return cache.set(key, timezone.now(), 2592000)



import logging

from django.core.cache import cache
from django.utils import timezone


logger = logging.getLogger(__name__)


class BaseExtension(object):
    name = None

    def check_service(self, request):
        raise NotImplementedError


class TaskExtension(BaseExtension):
    name = 'taskextension'

    def check_service(self, request):
        key = 'livewatch_{0}'.format(self.name)
        self.run_task()

        watchdog_timestamp = cache.get(key)
        if watchdog_timestamp is None:
            return False

        watchdog_timeout = int(request.GET.get('timeout', 900))
        watchdog_timestamp_diff = (timezone.now() - watchdog_timestamp)
        watchdog_timestamp_diff = (
            watchdog_timestamp_diff.seconds + watchdog_timestamp_diff.days * 24 * 3600)

        if watchdog_timestamp_diff > watchdog_timeout:
            return False

        return True

    def run_task(self):
        raise NotImplementedError



from __future__ import absolute_import

from .base import TaskExtension
from ..tasks import livewatch_update_celery_task


class CeleryExtension(TaskExtension):
    name = 'celery'

    def run_task(self):
        key = 'livewatch_{0}'.format(self.name)
        livewatch_update_celery_task.delay(key)
